---- CONVERT XYZ TO QUADKEY

CREATE OR REPLACE FUNCTION public.convert_xyz_to_quadkey(x INTEGER, y INTEGER, z INTEGER)
RETURNS TEXT AS
$$
    DECLARE
        quadkey TEXT;
        quadkey_digit INTEGER;
        rx INTEGER;
        ry INTEGER;
    BEGIN
        FOR i IN 1..z LOOP
            rx = x % 2;
            x = x / 2;
            ry = y % 2;
            y = y / 2;
            IF rx = 0 AND ry = 0 THEN
                quadkey_digit = 0;
            ELSIF rx = 1 AND ry = 0 THEN
                quadkey_digit = 1;
            ELSIF rx = 0 AND ry = 1 THEN
                quadkey_digit = 2;
            ELSIF rx = 1 AND ry = 1 THEN
                quadkey_digit = 3;
            ELSE
                RAISE EXCEPTION 'Failure converting xyz to Quadkey';
            END IF;
            quadkey = CONCAT(CAST(quadkey_digit AS CHAR), quadkey);
        END LOOP;
        RETURN quadkey;
    END;
$$
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE;
COMMENT ON FUNCTION public.convert_xyz_to_quadkey IS 'Converts a tile identifier from the format x,y,z to a Quadkey.';

---- CONVERT QUADKEY TO XYZ

CREATE OR REPLACE FUNCTION public.convert_quadkey_to_xyz(quadkey TEXT)
RETURNS SETOF INTEGER AS
$$
    DECLARE
        x INTEGER := 0;
        y INTEGER := 0;
        z INTEGER := 0;
        quadkey_digit TEXT;
    BEGIN
        FOR i IN 1..LENGTH($1) LOOP
            z = z + 1;
            quadkey_digit = SUBSTRING(quadkey, i, 1);
            IF quadkey_digit = '0' THEN
                x = x * 2; y = y * 2;
            ELSIF quadkey_digit = '1' THEN
                x = x * 2 + 1; y = y * 2;
            ELSIF quadkey_digit = '2' THEN
                x = x * 2; y = y * 2 + 1;
            ELSIF quadkey_digit = '3' THEN
                x = x * 2 + 1; y = y * 2 + 1;
            ELSE
                RAISE EXCEPTION 'Failure: a Quadkey can only contain numbers from `0` to `3`.';
            END IF;
        END LOOP;
        RETURN QUERY VALUES (x), (y), (z);
    END
$$
LANGUAGE plpgsql IMMUTABLE PARALLEL SAFE;
COMMENT ON FUNCTION public.convert_quadkey_to_xyz IS 'Converts a tile identifier from the format Quadkey to x,y,z.';
